import { Component, ElementRef, OnInit, ViewChild, } from '@angular/core';
import { ChatPage } from '../../aigc/chat/chat.page';
import { ParseObject } from './results-post';

import { CameraPagePage } from '../camera-page/camera-page.page';
import * as faceapi from '@vladmandic/face-api';
import Parse from "parse";
Parse.initialize("dev");
Parse.serverURL = "http://web2023.fmode.cn:9999/parse";

@Component({
  selector: 'app-results',
  templateUrl: './results.page.html',
  styleUrls: ['./results.page.scss'],
})
export class ResultsPage implements OnInit {
  @ViewChild('video', { static: true }) videoElement: ElementRef | any;
  @ViewChild('canvas', { static: true }) canvas: ElementRef | any;
  imageUrl: string=""; // 定义一个属性来存储图片URL
 static currentObjectId: string = 'zzzzz'; // 类成员变量，用于存储objectId
  serverURL:string="http://localhost:8100"

  constructor() { }

  ngOnInit() {
    this.runFaceAnanlysis();
    this.imageUrl = CameraPagePage.imageUrl || '';

  }

  onFileChange(event: any) {
    const file = event.target.files[0];
    if (file) {
      const reader = new FileReader();
      reader.onload = (e: any) => {
        const img = new Image();
        img.src = e.target.result;
        img.onload = () => {
          const canvasEl = this.canvas.nativeElement;
          const context = canvasEl.getContext('2d');

          canvasEl.width = img.width;
          canvasEl.height = img.height;
          context.drawImage(img, 0, 0);

          setTimeout(() => {
            this.runFaceAnanlysis();
          }, 2000);
        };
        document.getElementById('face')?.setAttribute('src', e.target.result);
      };
      reader.readAsDataURL(file);
    }
  }
  runFaceAnanlysis() {
    console.log(faceapi);

    const calculateSanTing = (positions: any) => {
      // 计算三庭的比例
      const leftEye = positions[36]; // 左眼角
      const rightEye = positions[42]; // 右眼角
      const noseBase = positions[33]; // 鼻尖
      const mouth = positions[48]; // 嘴巴中心

      const foreheadHeight = noseBase.x - leftEye.x; // 水平距离
      const noseLength = rightEye.x - leftEye.x; // 水平距离
      const chinHeight = mouth.x - noseBase.x; // 水平距离

      return {
        foreheadHeight: parseFloat(foreheadHeight.toFixed(2)),
        noseLength: parseFloat(noseLength.toFixed(2)),
        chinHeight: -parseFloat(chinHeight.toFixed(2)),
      };
    };

    const calculateWuYan = (positions: any) => {
      // 计算五眼的比例
      const leftEyebrow = positions[17]; // 左眉毛上角
      const rightEyebrow = positions[22]; // 右眉毛上角

      const eyeSpan = Math.abs(leftEyebrow.x - rightEyebrow.x);
      return parseFloat((eyeSpan * 5).toFixed(2));
    };

    const calculateFaceShape = (positions: any) => {
      // 计算脸型
      const jawOutline = positions.slice(0, 17); // 下巴轮廓
      const cheekLeft = positions[29]; // 左颧骨
      const cheekRight = positions[30]; // 右颧骨
      const foreheadLeft = positions[21]; // 左额头
      const foreheadRight = positions[22]; // 右额头

      const cheekWidth = Math.sqrt((cheekRight.x - cheekLeft.x) ** 2 + (cheekRight.y - cheekLeft.y) ** 2);
      const foreheadWidth = Math.sqrt((foreheadRight.x - foreheadLeft.x) ** 2 + (foreheadRight.y - foreheadLeft.y) ** 2);
      const jawWidth = Math.sqrt((jawOutline[8].x - jawOutline[6].x) ** 2 + (jawOutline[8].y - jawOutline[6].y) ** 2);
      const faceLength = Math.sqrt((jawOutline[8].x - jawOutline[0].x) ** 2 + (jawOutline[8].y - jawOutline[0].y) ** 2);

      const cheekForeheadRatio = cheekWidth / foreheadWidth;
      const jawForeheadRatio = jawWidth / foreheadWidth;
      const faceAspectRatio = faceLength / foreheadWidth;

      if (faceAspectRatio > 1.5) {
        if (cheekForeheadRatio > 0.8 && cheekForeheadRatio < 1.2 && jawForeheadRatio < 0.8) {
          return '瓜子脸';
        } else {
          return '长形脸';
        }
      } else if (faceAspectRatio < 1) {
        return '宽脸';
      } else {
        if (cheekForeheadRatio > 0.8 && cheekForeheadRatio < 1.2 && jawForeheadRatio > 0.8) {
          return '心形脸';
        } else if (cheekForeheadRatio < 0.8 && jawForeheadRatio < 0.8) {
          return '菱形脸';
        } else {
          return '标准脸';
        }
      }
    };

    const calculateBrowShape = (positions: any) => {
      // 计算眉形
      const leftEyebrowLeft = positions[17]; // 左眉毛上角
      const leftEyebrowRight = positions[22]; // 左眉毛下角
      const rightEyebrowLeft = positions[25]; // 右眉毛上角
      const rightEyebrowRight = positions[31]; // 右眉毛下角

      const leftBrowWidth = Math.abs(leftEyebrowLeft.x - leftEyebrowRight.x);
      const rightBrowWidth = Math.abs(rightEyebrowLeft.x - rightEyebrowRight.x);
      const browWidth = Math.max(leftBrowWidth, rightBrowWidth);

      const leftBrowLength = Math.sqrt((leftEyebrowLeft.x - leftEyebrowRight.x) ** 2 + (leftEyebrowLeft.y - leftEyebrowRight.y) ** 2);
      const rightBrowLength = Math.sqrt((rightEyebrowLeft.x - rightEyebrowRight.x) ** 2 + (rightEyebrowLeft.y - rightEyebrowRight.y) ** 2);
      const browLength = Math.max(leftBrowLength, rightBrowLength);

      if (browWidth > 0.5 * browLength) {
        return '浓眉';
      } else if (browWidth < 0.3 * browLength) {
        return '细眉';
      } else {
        return '标准眉';
      }
    };

    const calculateLipShape = (positions: any) => {
      // 计算唇形
      const upperLipTop = positions[61]; // 上嘴唇顶部
      const upperLipBottom = positions[62]; // 上嘴唇底部
      const lowerLipTop = positions[63]; // 下嘴唇顶部
      const lowerLipBottom = positions[64]; // 下嘴唇底部

      if (upperLipBottom.y - upperLipTop.y > 10 && lowerLipBottom.y - lowerLipTop.y > 10) {
        return '厚唇';
      } else if (upperLipBottom.y - upperLipTop.y < 5 && lowerLipBottom.y - lowerLipTop.y < 5) {
        return '薄唇';
      } else if (upperLipBottom.y - upperLipTop.y > 10 && lowerLipBottom.y - lowerLipTop.y < 5) {
        return '微笑唇';
      } else if (upperLipBottom.y - upperLipTop.y < 5 && lowerLipBottom.y - lowerLipTop.y > 10) {
        return '嘟嘟唇';
      } else {
        return '标准唇型';
      }
    };

    const calculateEyeShape = (positions: any) => {
      // 计算眼睛形状
      const leftEyeInner = positions[33]; // 左眼内侧
      const leftEyeOuter = positions[39]; // 左眼外侧
      const rightEyeInner = positions[45]; // 右眼内侧
      const rightEyeOuter = positions[51]; // 右眼外侧

      const leftEyeLength = Math.sqrt((leftEyeInner.x - leftEyeOuter.x) ** 2 + (leftEyeInner.y - leftEyeOuter.y) ** 2);
      const rightEyeLength = Math.sqrt((rightEyeInner.x - rightEyeOuter.x) ** 2 + (rightEyeInner.y - rightEyeOuter.y) ** 2);
      const eyeLength = Math.max(leftEyeLength, rightEyeLength);

      if (eyeLength > 10 && leftEyeLength > rightEyeLength) {
        return '杏仁眼';
      } else if (eyeLength > 10 && leftEyeLength < rightEyeLength) {
        return '圆形眼';
      } else if (leftEyeInner.y > leftEyeOuter.y) {
        return '单眼皮';
      } else {
        return '双眼皮';
      }
    };

    const calculateNoseShape = (positions: any) => {
      // 计算鼻形
      const noseBase = positions[33]; // 鼻尖
      const noseTip = positions[34]; // 鼻尖
      const noseSide = positions[35]; // 鼻翼

      if (noseBase.y > noseTip.y && noseSide.x > noseTip.x) {
        return '鹰钩鼻';
      } else if (noseBase.y > noseTip.y && noseSide.x < noseTip.x) {
        return '蒜头鼻';
      } else if (noseBase.y < noseTip.y) {
        return '直鼻';
      } else {
        return '塌鼻';
      }
    };

    const calculateCheekBones = (positions: any) => {
      // 计算颧骨
      const cheekLeft = positions[29]; // 左颧骨
      const cheekRight = positions[30]; // 右颧骨

      if (cheekLeft.y > cheekRight.y) {
        return '颧骨高';
      } else {
        return '颧骨平';
      }
    };

    const calculateChin = (positions: any) => {
      // 计算下巴
      const chin = positions[60]; // 下巴中心

      if (chin.y < chin.x) {
        return '长下巴';
      } else if (chin.y > chin.x) {
        return '短下巴';
      } else {
        return '尖下巴';
      }
    };

    const calculateForehead = (positions: any) => {
      // 计算额头
      const foreheadLeft = positions[21]; // 左额头
      const foreheadRight = positions[22]; // 右额头

      if (foreheadLeft.x > foreheadRight.x) {
        return '宽额头';
      } else {
        return '窄额头';
      }
    };

    const calculateCheeksShape = (positions: any) => {
      // 计算脸颊形状
      const cheekLeft = positions[29]; // 左颧骨
      const cheekRight = positions[30]; // 右颧骨

      if (cheekLeft.y > cheekRight.y) {
        return '丰满脸颊';
      } else {
        return '瘦削脸颊';
      }
    };

    const run = async () => {
      // 加载模型
      await Promise.all([
        faceapi.nets.ssdMobilenetv1.loadFromUri('/assets/models'),
        faceapi.nets.faceLandmark68Net.loadFromUri('/assets/models'),
        faceapi.nets.faceRecognitionNet.loadFromUri('/assets/models'),
        faceapi.nets.ageGenderNet.loadFromUri('/assets/models'),
      ]);

      // const face1: any | null = document.getElementById('face'); //获取html页面上face的src中的img链接
      //  const imageUrl: string = face1!.getAttribute('src')!;

      const face1: any | null = document.getElementById('face'); //获取html页面上face的src中的img链接
      const imageUrl: string = CameraPagePage.imageUrl!;
      if(!imageUrl)
      {
        alert("未上传照片请重试");
      return;
      }
      const FaceName:string = "";

      let faceAIData = await faceapi
        .detectAllFaces(face1)
        .withFaceLandmarks()
        .withFaceDescriptors()
        .withAgeAndGender();

      console.log(faceAIData);

      const landmarks: any = faceAIData[0].landmarks;
      const positions: any = landmarks._positions;

      const sanTing = calculateSanTing(positions);
      const wuYan = calculateWuYan(positions);
      const faceShape = calculateFaceShape(positions);
      const browShape = calculateBrowShape(positions);
      const lipShape = calculateLipShape(positions);
      const eyeShape = calculateEyeShape(positions);
      const noseShape = calculateNoseShape(positions);
      const cheekBones = calculateCheekBones(positions);
      const chinShape = calculateChin(positions);
      const foreheadShape = calculateForehead(positions);
      const cheeksShape = calculateCheeksShape(positions);

      const FaceDataString = {
        "三庭比例": {
          "前庭": sanTing.foreheadHeight,
          "中庭": sanTing.noseLength,
          "下庭": sanTing.chinHeight,
        },
        "五眼比例": wuYan,
        "脸型": faceShape,
        "眉形": browShape,
        "唇形": lipShape,
        "眼睛形状": eyeShape,
        "鼻形": noseShape,
        "颧骨": cheekBones,
        "下巴": chinShape,
        "额头": foreheadShape,
        "脸颊": cheeksShape
      };
      const FaceDataJson = JSON.stringify(FaceDataString);

      console.log(`三庭比例: ${sanTing.foreheadHeight},${sanTing.noseLength}, ${sanTing.chinHeight}`);
      console.log(`五眼比例: ${wuYan}`);
      console.log(`脸型: ${faceShape}`);
      console.log(`眉形: ${browShape}`);
      console.log(`唇形: ${lipShape}`);
      console.log(`眼睛形状: ${eyeShape}`);
      console.log(`鼻形: ${noseShape}`);
      console.log(`颧骨: ${cheekBones}`);
      console.log(`下巴: ${chinShape}`);
      console.log(`额头: ${foreheadShape}`);
      console.log(`脸颊: ${cheeksShape}`);

      ResultsPage.currentObjectId = await SaveFace(FaceName, imageUrl, FaceDataJson,this.serverURL);
      const foreheadHeightElement = document.getElementById('foreheadHeight');
      const noseLengthElement = document.getElementById('noseLength');
      const chinHeightElement = document.getElementById('chinHeight');
      const wuYanElement = document.getElementById('wuYan');
      const faceShapeElement = document.getElementById('faceShape');
      const browShapeElement = document.getElementById('browShape');
      const lipShapeElement = document.getElementById('lipShape');
      const eyeShapeElement = document.getElementById('eyeShape');
      const noseShapeElement = document.getElementById('noseShape');
      const cheekBonesElement = document.getElementById('cheekBones');
      const chinShapeElement = document.getElementById('chinShape');
      const foreheadShapeElement = document.getElementById('foreheadShape');
      const cheeksShapeElement = document.getElementById('cheeksShape');

      if (foreheadHeightElement) foreheadHeightElement.textContent = `${sanTing.chinHeight}`;
      if (noseLengthElement) noseLengthElement.textContent = ` ${sanTing.noseLength}`;
      if (chinHeightElement) chinHeightElement.textContent = ` ${sanTing.chinHeight}`;
      if (wuYanElement) wuYanElement.textContent = `${wuYan}`;
      if (faceShapeElement) faceShapeElement.textContent = faceShape;
      if (browShapeElement) browShapeElement.textContent = browShape;
      if (lipShapeElement) lipShapeElement.textContent = lipShape;
      if (eyeShapeElement) eyeShapeElement.textContent = eyeShape;
      if (noseShapeElement) noseShapeElement.textContent = noseShape;
      if (cheekBonesElement) cheekBonesElement.textContent = cheekBones;
      if (chinShapeElement) chinShapeElement.textContent = chinShape;
      if (foreheadShapeElement) foreheadShapeElement.textContent = foreheadShape;
      if (cheeksShapeElement) cheeksShapeElement.textContent = cheeksShape;

      let chatPage = new ChatPage();
      chatPage.sendMessage();

      async function SaveFace(faceName: any, imageUrl: any, Data: any,severUrl:any): Promise<any> {
        let YCX: any = new ParseObject("MagicMirror");
        if (faceName === "") {
          YCX.set({
            face: "NULL",
            image: imageUrl,
            features: Data,
          });
        }
        else{
          YCX.set({
            face: faceName,
            image: imageUrl,
            features: Data,
          });
        }
        
        try {
          const savedYCX = await YCX.save();
          return savedYCX; // 返回保存后的YCX对象
        } catch (error) {
          console.error("保存失败：", error);
          throw error; // 抛出错误，以便调用者可以捕获它
        }
      }
    };

    run();//脸部计算
  }
}
